package sgc.portal.base.service;

import sgc.portal.base.model.ComData;
import sgc.portal.base.persistence.ComSearchMapper;
import sgc.portal.base.model.FileAll;
import sgc.portal.base.util.EasyHashMap;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;


/**
 * The type Com search service.
 *
 * @author mir(khjang @ sgc.co.kr)
 * @version 1.0.0
 * @since 2020. 8. 2.
 */
@Transactional
@Service
public class ComSearchService {

    @Autowired
    private ComSearchMapper comSearchMapper;

    /**
     * Select prg auth list.
     *
     * @param comdatas the comdatas
     * @return the list
     */
//사용자별프로그램 권한정보 조회
    public EasyHashMap<String, Object> selectPrgAuth(ComData comdatas) {
        return comSearchMapper.selectPrgAuth(comdatas);
    }

    /**
     * Select pymtdedc list.
     *
     * @param comdatas the comdatas
     * @return the list
     */
//지급공제항목
    public List<EasyHashMap<String, Object>> selectPymtdedc(ComData comdatas) {
        return comSearchMapper.selectPymtdedc(comdatas);
    }

    /**
     * Select acvm pool list.
     *
     * @param comdatas the comdatas
     * @return the list
     */
//일반직 업적평가Pool 조회
    public List<EasyHashMap<String, Object>> selectAcvmPool(ComData comdatas) {
        return comSearchMapper.selectAcvmPool(comdatas);
    }

    /**
     * Select rewa pr list.
     *
     * @param comdatas the comdatas
     * @return the list
     */
//상벌공통코드 조회
    public List<EasyHashMap<String, Object>> selectRewaPR(ComData comdatas) {
        return comSearchMapper.selectRewaPR(comdatas);
    }

    /**
     * Select com dtl list.
     *
     * @param comdatas the comdatas
     * @return the list
     */
//공통코드 조회(대분류코드와 설정코드1의 조건으로 공통코드 조회)
    public List<EasyHashMap<String, Object>> selectComDtl(ComData comdatas) {
        return comSearchMapper.selectComDtl(comdatas);
    }

    /**
     * Select com dtl 2 list.
     *
     * @param comdatas the comdatas
     * @return the list
     */
//공통코드 조회(대분류코드와 설정코드2의 조건으로 공통코드 조회)
    public List<EasyHashMap<String, Object>> selectComDtl2(ComData comdatas) {
        return comSearchMapper.selectComDtl2(comdatas);
    }

    /**
     * Select com dtl 4 list.
     *
     * @param comdatas the comdatas
     * @return the list
     */
//공통코드 조회(대분류코드와 설정코드4의 조건으로 공통코드 조회)
    public List<EasyHashMap<String, Object>> selectComDtl4(ComData comdatas) {
        return comSearchMapper.selectComDtl4(comdatas);
    }

    /**
     * Select com evdc list.
     *
     * @param comdatas the comdatas
     * @return the list
     */
//증빙코드정보 조회( 2020-04-12 mir)
    public List<EasyHashMap<String, Object>> selectComEvdc(ComData comdatas) {
        return comSearchMapper.selectComEvdc(comdatas);
    }

    /**
     * Select hdlg auth list.
     *
     * @param comdatas the comdatas
     * @return the list
     */
//업무 취급 권한 정보 조회( 2020-07-02 blue3819)
    public List<EasyHashMap<String, Object>> selectHdlgAuth(ComData comdatas) {
        return comSearchMapper.selectHdlgAuth(comdatas);
    }

    /**
     * Select com bank list.
     *
     * @param comdatas the comdatas
     * @return the list
     */
//은행정보 조회( 2020-07-02 mir)
    public List<EasyHashMap<String, Object>> selectComBank(ComData comdatas) {
        return comSearchMapper.selectComBank(comdatas);
    }

    /**
     * Select com acct list.
     *
     * @param comdatas the comdatas
     * @return the list
     */
//은행정계좌조회( 2020-07-02 mir)
    public List<EasyHashMap<String, Object>> selectComAcct(ComData comdatas) {
        return comSearchMapper.selectComAcct(comdatas);
    }

    /**
     * Select com map bsu cd list.
     *
     * @param comdatas the comdatas
     * @return the list
     */
//사업장 코드 조회( 2020-08-28 ax5656)
    public List<EasyHashMap<String, Object>> selectComMapBsuCd(ComData comdatas) {
        return comSearchMapper.selectComMapBsuCd(comdatas);
    }


    /**
     * Select ba lacl cd list.
     *
     * @param comdatas the comdatas
     * @return the list
     */
//사업운영의 집행예산코드 대분류 조회(2019-07-22 Mir)
    public List<EasyHashMap<String, Object>> selectBaLaclCd(ComData comdatas) {
        return comSearchMapper.selectBaLaclCd(comdatas);
    }

    /**
     * Select ba mdcl cd list.
     *
     * @param comdatas the comdatas
     * @return the list
     */
//사업운영의 집행예산코드 중분류 조회(2019-07-22 Mir)
    public List<EasyHashMap<String, Object>> selectBaMdclCd(ComData comdatas) {
        return comSearchMapper.selectBaMdclCd(comdatas);
    }

    /**
     * Select ba com exto std list.
     *
     * @param comdatas the comdatas
     * @return the list
     */
//표준집행예산코드 조회 (2019-07-30 Mir)
    public List<EasyHashMap<String, Object>> selectBaComExtoStd(ComData comdatas) {
        return comSearchMapper.selectBaComExtoStd(comdatas);
    }

    public List<FileAll> selectAllFileList(FileAll fileAll) {
        return comSearchMapper.selectAllFileList(fileAll);
    }

}
