package sgc.portal.com.auth.web;

import sgc.portal.com.auth.model.PrvtinfoFindHist;
import sgc.portal.com.auth.service.MaskingDesabledService;
import sgc.portal.base.util.EasyHashMap;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;

/**
 * <pre>
 * 시스템명   : Common
 * 프로그램ID : Masking Desabled
 * 프로그램명  : 개인정보 마스킹 비활성화
 * 설명 : 개인정보 마스킹 비활성화 권한 체크 > 마스킹 비활성화 이력 기록 > 마스킹 비활성화 가능 유무 Return
 * </pre>
 *
 * @author 이범근
 * @version 2020.01.13
 * @see MaskingDesabledService
 * @since JDK1.8
 */
@RestController
@RequestMapping("/com/auth")
public class MaskingDesabledController {

    @Autowired
    private MaskingDesabledService maskingDesabledService;

    /**
     * Check masking auth easy hash map.
     *
     * @param prvtinfoFindHist the prvtinfo find hist
     * @return the easy hash map
     */
    @RequestMapping(value = "/checkMaskingAuth", method = RequestMethod.POST)
    public EasyHashMap<String, String> checkMaskingAuth(@RequestBody PrvtinfoFindHist prvtinfoFindHist){
        EasyHashMap<String, String> ret = new EasyHashMap<>();
        ret.put("chkYn", maskingDesabledService.maskingDesabledCheck(prvtinfoFindHist));
        return ret;
    }


    /**
     * 개인 정보 조회 이력
     *
     * @param prvtinfoFindHist 기본 모델 제공
     * @return 개인 정보 조회 이력 목록
     */
    @RequestMapping(value = "/selectMaskingAuth", method = RequestMethod.POST)
    public List<EasyHashMap<String, Object>> selectPrvtinfoFindHist(@RequestBody PrvtinfoFindHist prvtinfoFindHist){
        return maskingDesabledService.selectPrvtinfoFindHist(prvtinfoFindHist);

    }

}
