/**
 * @target 메시지 처리 관련 공통함수
 */

/**
 * 메시지 처리 관련 공통함수를 제공하는 개체입니다.
 *
 * @type {object}
 * @class msgLib
 * @namespace msgLib
 */
var msgLib = {};

/**
 * 메시지 리파지터리.<br>
 * 최초 1회에 SessionStorage에 저장해서 시스템 전체에서 공유하는 방식으로 사용됩니다.
 *
 * @author SONG WONSEOK
 * @date 2017. 06. 29.
 * @memberOf msgLib
 */

msgLib.properties = [
    { key : "exception.sql", koMsg : "데이터베이스 처리 과정 중에 에러가 발생 했습니다.", enMsg : ""},
    { key : "exception.sql.271", koMsg : "입력 작업중입니다. 잠시 대기해 주십시요.", enMsg : ""},
    { key : "exception.sql.244", koMsg : "자료 조회 작업중입니다. 잠시 대기해 주십시요.", enMsg : ""},
    { key : "exception.sql.268", koMsg : "중복 입력된 값이 존재합니다. 입력(필수) 값을 확인하십시요.", enMsg : ""},
    { key : "exception.sql.691", koMsg : "선행 입력값이 누락되었습니다. 확인후 작업하십시요.", enMsg : ""},
    { key : "exception.sql.284", koMsg : "1개이상의 자료 존재로 작업중 오류가 발생하였습니다.", enMsg : ""},
    { key : "exception.session.invalid", koMsg : "세션이 종료되었습니다. 다시 로그인하세요.", enMsg : ""},
    { key : "exception.authentication.invalid", koMsg : "아이디 또는 비밀번호가 다릅니다. 다시 로그인하세요.", enMsg : ""},
    { key : "exception.authentication.invalidOver", koMsg : "비밀번호를 5회이상 틀렸습니다. 하단의 PW 초기화 기능을 이용하세요.", enMsg : ""},
    { key : "exception.authentication.expire", koMsg : "패스워드를 변경한지 3개월이 지났습니다. 패스워드 변경후 로그인 가능합니다.", enMsg : ""},
    { key : "exception.unsupported.grid.status", koMsg : "지원하지 않는 그리드 로우 타입입니다.", enMsg : ""},
    { key : "exception.storage", koMsg : "파일 업로드 중 에러가 발생했습니다. 관리자에게 문의하세요. [{0}]", enMsg : ""},
    { key : "exception.storage.delete", koMsg : "첨부파일을 삭제하는데 실패했습니다. 관리자에게 문의하세요. [{0}]", enMsg : ""},
    { key : "exception.storage.not.allowd.file", koMsg : "업로드 허용된 첨부파일이 아닙니다. [{0}]", enMsg : ""},
    { key : "exception.password.invalid", koMsg : "사용자 정보가 일치 하지 않습니다.", enMsg : ""},
    { key : "exception.password.lenght", koMsg : "비밀번호는 {0}자이상 16자이하로 입력하세요", enMsg : ""},
    { key : "exception.password.chk1", koMsg : "비밀번호는 영소문자와 숫자의 조합으로만 입력해 주세요.", enMsg : ""},
    { key : "exception.password.chk2", koMsg : "기존 비밀번호와 변경 비밀번호가 동일합니다.", enMsg : ""},
    { key : "exception.password.chk3", koMsg : "변경 비밀번호와 변경 비밀번호 확인이 일치하지 않습니다.", enMsg : ""},

    //시스템 메세지
    { key : "sysm.0006", koMsg : "사용자 세션이 만료되었거나 접근 경로가 올바르지 않습니다.\n다시 로그인해주십시오.", enMsg : ""},

    //유효성 검사 관련 메시지
    { key : "valid.0001", koMsg : "서버 측 유효성 검사에 실패했습니다.", enMsg : "Server-side validation failed."},
    { key : "valid.0002", koMsg : "중복된 데이터가 이미 존재합니다.", enMsg : "Duplicate data already exists."},
    { key : "valid.0003", koMsg : "한글을 입력할 수 없는 항목입니다. ({0})", enMsg : "You can't enter Korean characters. ({0})"},
    { key : "valid.0004", koMsg : "이메일 주소의 형식이 올바르지 않습니다. ({0})", enMsg : "The email address is not formatted correctly. ({0})"},
    { key : "valid.0005", koMsg : "도메인 형식이 올바르지 않습니다. ({0})", enMsg : ""},
    { key : "valid.0006", koMsg : "정수 형식이 올바르지 않습니다. ({0})", enMsg : ""},
    { key : "valid.0007", koMsg : "실수 형식이 올바르지 않습니다. ({0})", enMsg : ""},
    { key : "valid.0008", koMsg : "입력값이 허용 범위를 벗어났습니다. ({0})", enMsg : ""},
    { key : "valid.0009", koMsg : "날짜의 형식이 올바르지 않습니다. ({0})", enMsg : ""},
    { key : "valid.0010", koMsg : "주민등록번호 형식이 올바르지 않습니다. ({0})", enMsg : ""},
    { key : "valid.0011", koMsg : "법인등록번호 형식이 올바르지 않습니다. ({0})", enMsg : ""},
    { key : "valid.0012", koMsg : "사업자등록번호 형식이 올바르지 않습니다. ({0})", enMsg : ""},
    { key : "valid.0013", koMsg : "영문자로 시작되어야 하고 영문이나 숫자만 입력할 수 있는 항목입니다. ({0})", enMsg : ""},
    { key : "valid.0014", koMsg : "필수입력 항목입니다. ({0})", enMsg : ""},
    { key : "valid.0015", koMsg : "최소 {1}자 이상 입력해야 합니다. ({0})", enMsg : ""},
    { key : "valid.0016", koMsg : "최소 {1}바이트 이상 입력해야 합니다. ({0})", enMsg : ""},
    { key : "valid.0017", koMsg : "최대 {1}자까지만 입력할 수 있습니다. ({0})", enMsg : ""},
    { key : "valid.0018", koMsg : "최대 {1}바이트까지만 입력할 수 있습니다. ({0})", enMsg : ""},
    { key : "valid.0019", koMsg : "필수입력 항목입니다.", enMsg : ""},
    { key : "valid.0020", koMsg : "최소 {0}자 이상 입력해야하는 항목입니다.", enMsg : ""},
    { key : "valid.0021", koMsg : "최대 {0}자까지만 입력할 수 있습니다.", enMsg : ""},
    { key : "valid.0022", koMsg : "최대 {0}바이트까지만 입력할 수 있습니다.", enMsg : ""},
    { key : "valid.0023", koMsg : "한글을 입력할 수 없는 항목입니다.", enMsg : ""},
    { key : "valid.0024", koMsg : "이메일 주소의 형식이 올바르지 않습니다.", enMsg : "The email address is not formatted correctly."},
    { key : "valid.0025", koMsg : "날짜의 형식이 올바르지 않습니다.", enMsg : ""},
    { key : "valid.0026", koMsg : "정수 형식이 올바르지 않습니다.", enMsg : ""},
    { key : "valid.0027", koMsg : "실수 형식이 올바르지 않습니다.", enMsg : ""},
    { key : "valid.0028", koMsg : "시작일자와 종료일자가 올바르지 않습니다.", enMsg : ""},
    { key : "valid.0029", koMsg : "변경된 데이터가 존재하지 않습니다.", enMsg : "No changed data exists."},
    { key : "valid.0030", koMsg : "숫자만 입력할 수 있는 항목입니다.", enMsg : ""},
    { key : "valid.0031", koMsg : "조회기간의 시작시점이 종료시점보다 뒤입니다.", enMsg : ""},
    { key : "valid.0032", koMsg : "{0} (전산실에 문의하시기 바랍니다.)", enMsg : ""},
    { key : "valid.0033", koMsg : "{0} 가능합니다.", enMsg : ""},
    { key : "valid.0042", koMsg : "사용자 정보 체크 후 선택해주십시오.", enMsg : ""},
    { key : "valid.0043", koMsg : "권한사용자 정보 체크 후 선택해주십시오.", enMsg : ""},
    { key : "valid.0044", koMsg : "사용자에 추가할 직원을 선택해주십시오.", enMsg : ""},
    { key : "valid.0045", koMsg : "이미 시스템에 등록된 사용자입니다: {0}", enMsg : ""},
    { key : "valid.0049", koMsg : "이미 추가된 수신자입니다. 체크 후 추가해주십시오.", enMsg : ""},
    { key : "valid.0050", koMsg : "시작일은 필수 항목입니다.", enMsg : ""},
    { key : "valid.0051", koMsg : "종료일은 필수 항목입니다.", enMsg : ""},
    { key : "valid.0052", koMsg : "시작일은 종료일 이후일 수 없습니다.", enMsg : ""},
    { key : "valid.0057", koMsg : "완성형 한글을 입력하세요. 다음 문자는 입력할 수 업습니다. [{0}]", enMsg : "" },
    { key : "valid.0058", koMsg : "필수입력항목은 수정이 불가능 합니다 .", enMsg : ""},
    { key : "valid.0059", koMsg : "이미 추가된 항목입니다. 확인 후 추가 해주십시오.", enMsg : ""},
    { key : "valid.0060", koMsg : "삭제는 한 행만 선택이 가능합니다.", enMsg : ""},
    { key : "valid.0061", koMsg : "선택은 한 행만 선택이 가능합니다.", enMsg : "Only one row can be selected."},
    { key : "valid.0062", koMsg : "저장할 정보가 존재하지 않습니다. 선택해 주세요.", enMsg : ""},
    { key : "valid.0063", koMsg : "하위 메뉴 및 프로그램이 존재합니다. \n 하위메뉴 및 프로그램을 삭제 후 삭제를 해주십시오.", enMsg : ""},
    { key : "valid.0064", koMsg : "프로그램을 선택 했습니다. \n 프로그램은 삭제가 불가능 합니다.", enMsg : ""},
    { key : "valid.0065", koMsg : "메뉴를 선택 했습니다. \n 메뉴는 삭제가 불가능 합니다.", enMsg : ""},
    { key : "valid.0066", koMsg : "관리구분 없는 항목은 추가 할수 없습니다.", enMsg : ""},
    { key : "valid.0070", koMsg : "이미 승인요청된 데이터가 존재합니다.", enMsg : ""},
    { key : "valid.0071", koMsg : "이미 처리된 데이터가 존재합니다.", enMsg : ""},
    { key : "valid.0072", koMsg : "사원을 선택해 주세요.", enMsg : ""},
    { key : "valid.0078", koMsg : "{0} 데이터가 존재하지 않습니다.", enMsg : "{0} The data does not exist."},
    { key : "valid.0079", koMsg : "삭제 대상 첨부파일이 존재하지 않습니다.", enMsg : ""},
    { key : "valid.0080", koMsg : "첨부파일 업로드에 실패를 했습니다. 첨부파일을 확인해 주세요.", enMsg : ""},
    { key : "valid.0083", koMsg : "메뉴를 먼저 선택 해 주세요.", enMsg : ""},
    { key : "valid.0090", koMsg : "{0}는 {1}보다 이후 일 수 없습니다. \n 확인 해 주세요.", enMsg : ""},
    { key : "valid.0096", koMsg : "{0}를 선택 후 추가버튼을 선택해 주세요.", enMsg : ""},
    { key : "valid.0097", koMsg : "{0} 먼저 저장하십시오.", enMsg : "" },
    { key : "valid.0098", koMsg : "{0} 필수값입니다.", enMsg : "{0} Required value." },
    { key : "valid.0099", koMsg : "{0} 잘못되었습니다.", enMsg : "" },
    { key : "valid.0100", koMsg : "{0} 선택하십시오.", enMsg : "Choose the {0}" },
    { key : "valid.0101", koMsg : "{0} 삭제를 할 수 없습니다.", enMsg : "" },
    { key : "valid.0102", koMsg : "{0} {1} 보다 {2} 수 없습니다.", enMsg : "" },
    { key : "valid.0108", koMsg : "{0} 일치하지 않습니다. 확인 해 주세요.", enMsg : "" },
    { key : "valid.0110", koMsg : "{0} 완료되었습니다.", enMsg : "" },
    { key : "valid.0111", koMsg : "{0} {1}하십시오.", enMsg : "{1} the {0}. " },
    { key : "valid.0112", koMsg : "{0} {1} 처리할 수 없습니다.", enMsg : "" },
    { key : "valid.0113", koMsg : "{0} 존재해야 합니다.", enMsg : "" },
    { key : "valid.0114", koMsg : "{0}인 경우만 선택이 가능합니다.", enMsg : "" },
    { key : "valid.0115", koMsg : "{0}의 내용을 모두 확인 후에 {1} 처리가 가능합니다.", enMsg : "" },
    { key : "valid.0116", koMsg : "{0}는 {1} 처리를 할 수 없습니다.", enMsg : "" },
    { key : "valid.0117", koMsg : "{0} 또는 {1}을(를) 입력 해 주세요.", enMsg : "" },
    { key : "valid.0118", koMsg : "{0}이(가) 0건입니다. {1}을(를) 선택해주세요.", enMsg : "" },
    { key : "valid.0124", koMsg : "{0} {1} 할 수 없습니다.", enMsg : "" },
    { key : "valid.0125", koMsg : "시간 형식이 올바르지 않습니다.({0})", enMsg : "" },
    { key : "valid.0134", koMsg : "{0}은 {1}을 초과할 수 없습니다.\n 다시 선택해 주세요.", enMsg : "" },
    { key : "valid.0136", koMsg : "{0} {1} {2} 필요합니다.", enMsg : ""},
    { key : "valid.0139", koMsg : "{0} {1} 처리할 수 없습니다. {2} 선택하십시오.", enMsg : "" },
    { key : "valid.0142", koMsg : "전화번호 형식이 올바르지 않습니다.", enMsg : ""},
    { key : "valid.0143", koMsg : "첨부파일이 존재하지 않습니다.", enMsg : ""},
    
    //처리 관련 메시지
    { key : "crud.0001", koMsg : "처리 하시겠습니까?", enMsg : "Do you want to deal with it?" },
    { key : "crud.0002", koMsg : "처리 완료하였습니다.", enMsg : "Processing is complete." },
    { key : "crud.0003", koMsg : "추가 : {0}건\n수정 : {1}건\n삭제 : {2}건\n처리를 진행 하시겠습니까?", enMsg : "" },
    { key : "crud.0004", koMsg : "추가/수정/삭제 등의 작업이 진행중입니다.\n조회를 진행하시겠습니까?", enMsg : "An operation such as add/edit/delete is in progress. Do you want to proceed with the search?" },
    { key : "crud.0005", koMsg : "권한을 선택해주십시오.", enMsg : "" },
    { key : "crud.0006", koMsg : "사용중인 정보는 삭제를 할수 없습니다.", enMsg : "" },
    { key : "crud.0007", koMsg : "기존 정보가 존재합니다,\n 기존 정보를 변경하시겠습니까?", enMsg : "" },
    { key : "crud.0008", koMsg : "{0}인 경우만 사용가능한 기능입니다.", enMsg : "" },
    { key : "crud.0009", koMsg : "선택된 내역이 없습니다. 확인후 다시 작업하세요.", enMsg : "There are no items selected. Please check and try again." },
    { key : "crud.0010", koMsg : "추가/수정/삭제 등의 작업이 진행중입니다.\n 텝 이동 하시겠습니까?", enMsg : "An action is in progress, such as add/edit/delete.\n Do you want to move a step?" },
    { key : "crud.0014", koMsg : "{0}건 {1} 처리를 진행 하시겠습니까?", enMsg : "" },
    { key : "crud.0015", koMsg : "{0} 처리 가능한 건이 없습니다. 확인 후 진행해주세요.", enMsg : "" },
    { key : "crud.0020", koMsg : "{0} 처리를 진행 하시겠습니까?", enMsg : "" },
    { key : "crud.0024", koMsg : "{0} 형식이 잘못되었습니다. 확인해주세요.", enMsg : "" },
    { key : "crud.0028", koMsg : "처리할 수 없습니다. 값을 확인해 보세요.", enMsg : "" },
    { key : "crud.0034", koMsg : "{0} 값을 먼저 선택하세요.", enMsg : "" },
    { key : "crud.0039", koMsg : "하위 데이터가 존재하여 삭제할 수 없습니다.", enMsg : "" },
    { key : "crud.0044", koMsg : "{0} 확인해주세요.", enMsg : "" },
    { key : "crud.0062", koMsg : "선택하지 않은 항목이 있습니다. 확인해주세요.", enMsg : "" },
    { key : "crud.0078", koMsg : "첨부파일을 삭제 하시겠습니까?", enMsg : "" },
    { key : "crud.0083", koMsg : "동일한 {0}이(가) 있습니다.", enMsg : "" },
    { key : "crud.0144", koMsg : "{0} {1} 하시겠습니까?", enMsg : "Do you want to {0} {1} with it?" },
    { key : "crud.0200", koMsg : "기준정보에 등록되지 않은 공종명 입니다.", enMsg : "기준정보에 등록되지 않은 공종명 입니다." },
    { key : "crud.0201", koMsg : "동일한 {0}을(를) 선택하세요.", enMsg : "Please select the same {0}." },
    
    //내부결재 메시지
    { key : "aprv.0001", koMsg : "상신 하시겠습니까?", enMsg : "Do you want to draft it?"},
    { key : "aprv.0002", koMsg : "승인 하시겠습니까?", enMsg : "Do you want to approval it?"},
    { key : "aprv.0003", koMsg : "반려 하시겠습니까?", enMsg : "Do you want to return it?"},
    { key : "aprv.0004", koMsg : "취소 하시겠습니까?", enMsg : "Do you want to cancle the drafed document?"},
    { key : "aprv.0005", koMsg : "상신 완료", enMsg : "Processed document. [Draft]"},
    { key : "aprv.0006", koMsg : "승인 완료", enMsg : "Processed document. [Approval]"},
    { key : "aprv.0007", koMsg : "반려 완료", enMsg : "Processed document. [Return]"},
    { key : "aprv.0008", koMsg : "상신 된 내역 입니다.", enMsg : "There is the drafed document"},
    { key : "aprv.0009", koMsg : "승인 된 내역 입니다.", enMsg : "There is the approved document"},
    { key : "aprv.0010", koMsg : "반려 된 내역 입니다.", enMsg : "There is the returned document"},
    { key : "aprv.0011", koMsg : "최종 승인 된 내역 입니다.", enMsg : "There is the final approved document"},
    { key : "aprv.0012", koMsg : "승인 내역이 아닙니다.", enMsg : "There is not the approved document"},
    { key : "aprv.0013", koMsg : "현 결재자가 아닙니다.", enMsg : "There is not current approver"},
    { key : "aprv.0014", koMsg : "최종 승인 완료된 내역 입니다.", enMsg : "There is the final approved document"},
    { key : "aprv.0015", koMsg : "결재라인을 추가 하세요.", enMsg : "Add approval line"},
    { key : "aprv.9998", koMsg : "완료", enMsg : "Complete"},
    { key : "aprv.9999", koMsg : "오류", enMsg : "Error"},
    
    //ALERT TITLE 메시지
    { key : "alert.0001", koMsg : "데이터확인", enMsg : "DATA CONFIRM"},
    { key : "alert.0002", koMsg : "데이터저장", enMsg : "DATA SAVE"},
    { key : "alert.0003", koMsg : "데이터결과", enMsg : "DATA RESULT"},
    { key : "alert.0004", koMsg : "상신", enMsg : "DRAFT"},
    { key : "alert.0005", koMsg : "승인", enMsg : "APPROVAL"},
    { key : "alert.0006", koMsg : "반려", enMsg : "RETURN"},
    { key : "alert.0007", koMsg : "취소", enMsg : "CANCLE"},
    { key : "alert.0008", koMsg : "{0} 일자 공사일보를 복사 하시겠습니까?", enMsg : "Copy the construction report dated {0} ?"},
    { key : "alert.0009", koMsg : "합의요청", enMsg : "CONSENSUS REQUEST"},
    { key : "alert.0010", koMsg : "합의", enMsg : "CONSENSUS"},
    { key : "alert.0011", koMsg : "작업확인서에 합의 하시겠습니까?", enMsg : "Do you want to agree to a confirmation of work?"},
    { key : "alert.0012", koMsg : "작업지시서에 승인 하시겠습니까?", enMsg : "Do you want to approve the work order?"},
    
    //POPUP TITLE 메시지
    { key : "title.0001", koMsg : "사원 정보", enMsg : "USER INFORMATION"},
    { key : "title.0002", koMsg : "부서 정보", enMsg : "DEPT. INFORMATION"},
    { key : "title.0003", koMsg : "상신", enMsg : "DRAFT"},
    { key : "title.0004", koMsg : "승인", enMsg : "APPROVAL"},
    { key : "title.0005", koMsg : "반려", enMsg : "RETURN"},
    { key : "title.0006", koMsg : "현장 정보", enMsg : "PROJECT"},
    { key : "title.0007", koMsg : "공종 선택", enMsg : "SELECT DISCIPLINE"},
    { key : "title.0008", koMsg : "장비 선택", enMsg : "SELECT EQUIPMENT"},
    { key : "title.0009", koMsg : "자재 선택", enMsg : "SELECT MATERIAL"},
    { key : "title.0010", koMsg : "공종/협력업체 선택", enMsg : "SELECT DISCIPLINE/SUBCONTRACTOR"},
    { key : "title.0011", koMsg : "전일기록 가져오기", enMsg : "COPY THE PREVIOUS DAY"},
    { key : "title.0012", koMsg : "작업지시서", enMsg : "WORK ORDER"},
    { key : "title.0013", koMsg : "작업확인서", enMsg : "COMPLETION OF WORK ORDER"},
    { key : "title.0014", koMsg : "작업지시서 전송", enMsg : "SEND WORK ORDER"},
    { key : "title.0015", koMsg : "공사일보", enMsg : "Construction Day" },
    { key : "title.0016", koMsg : "공종/협력업체 검색", enMsg : "SEARCH DISCIPLINE/SUBCONTRACTOR"},
    { key : "title.0017", koMsg : "결재진행이력", enMsg : "APPROVAL PROGRESS RECORD"},
    { key : "title.0018", koMsg : "메일 전송 이력", enMsg : "MAIL TRANSMISSION HISTORY"},
    { key : "title.0019", koMsg : "협력업체", enMsg : "SUBCONTRACTOR"},
    { key : "title.0020", koMsg : "미리보기", enMsg : "PREVIEW"},
    { key : "title.0021", koMsg : "파일첨부", enMsg : "FILE ATTACHMENTS"},
    { key : "title.0022", koMsg : "이미지 미리보기", enMsg : "PREVIEWING IMAGES"},
    { key : "title.0023", koMsg : "작업지시서 공유", enMsg : "SHARE WORK ORDER"},
    { key : "title.0024", koMsg : "대여금 자료 복사하기", enMsg : "COPY THE PREVIOUS LOAN DATA"},
    { key : "title.0025", koMsg : "차입금 자료 복사하기", enMsg : "COPY THE PREVIOUS DEBT DATA"},
    { key : "title.0026", koMsg : "보증정보 자료 복사하기", enMsg : "COPY THE PREVIOUS GUARANTEE DATA"},
    
    //ERROR TITLE 메시지
    { key : "error.0001", koMsg : "기준정보에 등록되지 않은 {0} 입니다.", enMsg : "This is a {0} that is not listed in the reference information." },
    { key : "error.0002", koMsg : "비고를 확인하세요.", enMsg : "See Remarks." },
    { key : "error.0003", koMsg : "선택 된 현장에 위도,경도 정보가 없습니다.", enMsg : "The selected field does not have latitude,longitude information." },
    { key : "error.0004", koMsg : "공사일보에서 사용 된 공종은 삭제 할 수 없습니다.", enMsg : "You can't delete the type of construction used in a construction report." },
    { key : "error.0005", koMsg : "공사일보에서 사용 된 장비는 삭제 할 수 없습니다.", enMsg : "You can't delete used equipment from a construction report." },
    { key : "error.0006", koMsg : "공사일보에서 사용 된 자재는 삭제 할 수 없습니다.", enMsg : "You can't delete materials used in a construction report." },
    
];